/* Copyright 2017 Kristofer Björnson
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/** @package TBTKcalc
 *  @file AbstractHoppingAmplitudeFilter.h
 *  @brief Filters HoppingAmplitudes using an IndexFilter.
 *
 *  @author Kristofer Björnson
 */

#ifndef COM_DAFER45_TBTK_INDEX_BASED_HOPPING_AMPLITUDE_FILTER
#define COM_DAFER45_TBTK_INDEX_BASED_HOPPING_AMPLITUDE_FILTER

#include "AbstractHoppingAmplitudeFilter.h"
#include "AbstractIndexFilter.h"

namespace TBTK{

class IndexBasedHoppingAmplitudeFilter : public AbstractHoppingAmplitudeFilter{
public:
	/** Constructor. */
	IndexBasedHoppingAmplitudeFilter(
		const AbstractIndexFilter &indexFilter
	);

	/** Copy constructor. */
	IndexBasedHoppingAmplitudeFilter(
		const IndexBasedHoppingAmplitudeFilter &indexBasedHoppingAmplitudeFilter
	);

	/** Move constructor. */
	IndexBasedHoppingAmplitudeFilter(
		IndexBasedHoppingAmplitudeFilter &&indexBasedHoppingAmplitudeFilter
	);

	/** Destructor. */
	virtual ~IndexBasedHoppingAmplitudeFilter();

	/** Assignment operator. */
	IndexBasedHoppingAmplitudeFilter& operator=(
		const IndexBasedHoppingAmplitudeFilter &indexBasedHoppingAmplitudeFilter
	);

	/** Move assignment operator. */
	IndexBasedHoppingAmplitudeFilter& operator=(
		IndexBasedHoppingAmplitudeFilter &&indexBasedHoppingAmplitudeFilter
	);

	/** Implementes AbstractHoppingAmplitudeFilter::clone(). */
	virtual IndexBasedHoppingAmplitudeFilter* clone() const;

	/** Implementes AbstractHoppingAmplitudeFilter::isIncluded(). */
	virtual bool isIncluded(
		const HoppingAmplitude &hoppingAmplitude
	) const;
private:
	/** Index filter. */
	AbstractIndexFilter *indexFilter;
};

}; //End of namesapce TBTK

#endif
